unit BELib;

interface

uses
  Windows, Messages, SysUtils, Classes, Forms, ShlObj, ComObj, ActiveX,
  ShellApi, FileCtrl, Graphics, StdCtrls, Math, Dialogs, Controls, Registry;

procedure MoveMainFormCenter(AForm, MainForm: TForm); overload;
procedure MoveMainFormCenter(AHwnd: THandle; MainForm: TForm); overload;
function IsInstalled(S: String): Boolean;
function MsgBox(Msg: String; Flags: Word): Integer;
function DeleteLastSeparator(S: String): String;
function GetSystemFolderName(nFolder: Integer): String;
function CreateShortcut(szFileName, Title: string; nFolder: Integer; Status: Integer): Boolean;
function IniName: String;
function PointToStr(Pt: TPoint): String;
function StrToPoint(Str: String): TPoint;
function ShortToLongFileName(FileName: String): String;
function AppPath: String;
procedure OpenFolder(F: String);
function ShellExec(const CmdLine: String): Longint;
function AppVer: String;
function StrToRect(Str: String): TRect;
function RectToStr(Rect: TRect): String;
function GetFileNameByShortCutLnk(LnkName: WideString): String;
function FontStylesToString(Styles: TFontStyles): String;
function StringToFontStyles(Styles: String): TFontStyles;
function CenteredInputQuery(const ACaption, APrompt: string; var Value: string; MainForm: TForm): Boolean;
function ActivatePrevInstance(const MainFormClass, ATitle: string): HWnd;
procedure LockForm(AForm: TForm);
procedure UnLockForm(AForm: TForm);
procedure SetComObject;
procedure DelComObject;
function UsingShellExtention: Boolean;


var
  FMainFormLockCount: Integer = 0;

implementation

procedure MoveMainFormCenter(AForm, MainForm: TForm);
begin
  if not (MainForm.HandleAllocated and AForm.HandleAllocated) then Exit;

  with AForm do
  begin
    Left := Max(10, MainForm.Left + (MainForm.Width  - Width)  div 2);
    Top  := Max(10, MainForm.Top  + (MainForm.Height - Height) div 2);

    if Top + Height + 10 > Screen.Height then
      Top := Screen.Height - Height - 10;
    if Left + Width + 10 > Screen.Width then
      Left := Screen.Width - Width - 10;

  end;
end;

procedure MoveMainFormCenter(AHwnd: THandle; MainForm: TForm);
var
  l, t: Integer;
  Placement: TWindowPlacement;
begin
  GetWindowPlacement(AHwnd, @Placement);
  with Placement.rcNormalPosition do
  begin
    l := Max(10, MainForm.Left + (MainForm.Width  - Right  + Left)  div 2);
    t := Max(10, MainForm.Top  + (MainForm.Height - Bottom + Top)  div 2);
  end;
  SetWindowPos(AHwnd, HWND_TOP, l, t, 0, 0, SWP_NOSIZE);
end;

// t@CpX̒ʂtH_ɃCXg[Ă邩ׂ
function IsInstalled(S: String): Boolean;
var
  Dat: array[0..511] of Char;
  P: PAnsiChar;
begin
  Result := False;
  if SearchPath( nil, PChar(S), nil, SizeOf( Dat ), Dat, P ) <> 0 then
    Result := True;
end;

// ėpbZ[W{bNX
function MsgBox(Msg: String; Flags: Word): Integer;
begin
  Result := Application.MessageBox(PChar(Msg), PChar(Application.Title), Flags);
end;

// t@CpX̍Ō"\"폜B
function DeleteLastSeparator(S: String): String;
begin
  Result := S;
  if Length( S) > 0 then
    if IsDelimiter('\', S, Length(S)) then
      Result := Copy( S, 1, Length(S) - 1);
end;


function GetSystemFolderName(nFolder: Integer): String;
// nFolder̒l
//   CSIDL_DESKTOPDIRECTORY : ޽į
//   CSIDL_PROGRAMS         : ƭ
//   CSIDL_STARTUP          : ı
//   CSIDL_SENDTO           : ÷ƭ́uv
var
  pIDList: PItemIDList;
  pszPath: array[0..MAX_PATH - 1] of Char;
begin
  Result := '';
  SHGetSpecialFolderLocation( 0, nFolder, pIDList); // ItemIDList 擾
  if SHGetPathFromIDList( pIDList, pszPath) then Result := pszPath; // ItemIDList tH_擾
end;

function CreateShortcut(szFileName, Title: string; nFolder: Integer; Status: Integer): Boolean;
// szFileName  = V[gJbg쐬tpXt@C
// nFolder = ĶĂ̍쐬ꏊ  CSIDL_DESKTOPDIRECTORY : ޽į
//                              CSIDL_PROGRAMS         : ƭ
//                              CSIDL_STARTUP          : ı
//                              CSIDL_SENDTO           : ÷ƭ
//                                                      uv
//Status: 1:o^@0:폜
var
  ppmem: IMalloc;
  Directory : array[0..MAX_PATH] of Char;
  szDirectory: string;
  UnknownObject: IUnknown;
  psl: IShellLink;
  ppf: IPersistFile;
  lpi: PItemIDList;
  wFileName : WideString;
begin
  Result := False;
  UnknownObject:= CreateComObject(CLSID_ShellLink);
  if not Assigned(UnknownObject) then Exit;
  psl:= UnknownObject as IShellLink;
  ppf:= UnknownObject as IPersistFile;
  if not Assigned(psl) or not Assigned(ppf) then Exit;
  if FAILED(ShGetMalloc(ppmem)) then exit;
  if FAILED(SHGetSpecialFolderLocation(0, nFolder, lpi)) then Exit;
  if not SHGetPathFromIDList(lpi, Directory)then
  begin
    if Assigned(lpi) then ppmem.Free(lpi);
    Exit;
  end;
  with psl do
  begin
    SetPath(PChar(szFileName));
    SetWorkingDirectory(PChar(ExtractFilePath(szFileName)));
  end;
  szDirectory:= strpas(Directory);
  wFileName:= szDirectory + '\' + Title + '.lnk';
  case Status of
    0: begin
         if not DeleteFile(wFileName) then
         begin
//           MsgBox('V[gJbg̍폜Ɏs܂B', MB_ICONSTOP);
           if Assigned(lpi) then ppmem.Free(lpi);
           Exit;
         end;
       end;
    1: ppf.Save(PWChar(wFileName), False);
  end;
  if Assigned(lpi) then ppmem.Free(lpi);
end;

function IniName: String;
begin
  Result := AppPath + 'BinEdit.ini';
end;

function PointToStr(Pt: TPoint): String;
begin
  Result := Format('%d,%d',[Pt.x, Pt.y]); 
end;

function StrToPoint(Str: String): TPoint;
var
  SL: TStringList;
begin
  SL := TStringList.Create;
  try
    SL.CommaText := Str;
    if SL.Count < 2 then
      Result := Point(0,0)
    else
      Result := Point(StrToIntDef(SL[0], 0), StrToIntDef(SL[1], 0));
  finally
    SL.Free;
  end;
end;


function GetLongFilePath(Dir: String): String;
  function GetRealName(Dir: String): String;
  var
    Sr: TSearchRec;
  begin
    if FindFirst(Dir, faAnyFile, Sr) = 0 then begin
      Result := Sr.Name;
      FindClose(Sr);
    end else
      Result := '';
  end;
var
  wDir, wDrv: String;
begin
  Result := '';
  if not DirectoryExists(Dir) then Exit;
  wDir := AnsiUpperCase(DeleteLastSeparator(Dir));
  wDrv := ExtractFileDrive(wDir);
  while wDir <> wDrv do begin
    Result := GetRealName(wDir) + '\' + Result;
    wDir := DeleteLastSeparator(ExtractFilePath(wDir));
  end;
  Result := wDrv + '\' + Result;
end;


// pX͒Z`̂܂
function ShortToLongFileName(FileName: String): String;
var
  SearchRec: TSearchRec;
begin
  if FindFirst(FileName, faAnyFile, SearchRec) = 0 then
    Result := GetLongFilePath(ExtractFileDir(FileName)) + SearchRec.Name
  else
    Result := FileName;
  FindClose(SearchRec);
end;


function AppPath: String;
begin
  Result := ExtractFilePath(Application.ExeName);
end;

procedure OpenFolder(F: String);
var
  si:TStartupInfo;
  pi:TProcessInformation;
begin
  if not DirectoryExists(F) then Exit;
  GetStartupInfo(si);
  si.wShowWindow := SW_SHOWNORMAL;
  CreateProcess(nil, PChar('EXPLORER.EXE '+F),nil,nil,false,0,nil,nil,si,pi);
end;

function ShellExec(const CmdLine: String): Longint;
  // R}hCvOƃp^ɕ
  procedure ExtractCommandLine(const CmdLine: string; var Prog, Parm: string);
  var
    WithQuote: Boolean;
      i, StartIndex, EndIndex: Integer;
    Cmd: string;
    SearchChar: Char;
  begin
    Cmd := Trim(CmdLine);
    Prog := '';
    Parm := '';
    if Length(Cmd) = 0 then Exit;
    WithQuote := Cmd[1] = '"';
    if WithQuote then
    begin
      SearchChar := '"';
      StartIndex := 2;
    end else
    begin
      SearchChar := ' ';
      StartIndex := 1;
    end;
    EndIndex := Length(Cmd);
    for i := StartIndex to Length(Cmd) do
    begin
      if (Cmd[i] = SearchChar) and (ByteType(Cmd, i) = mbSingleByte) then
      begin
        EndIndex := i;
        Break;
      end;
    end;
    if EndIndex = Length(Cmd) then
    begin
      if WithQuote then
        Prog := Copy(Cmd, 2, Length(Cmd) - 2)
      else
        Prog := Cmd;
    end else
    begin
      Prog := Copy(Cmd, 1, EndIndex);
      Parm := Trim(Copy(Cmd, EndIndex + 1, Length(Cmd) - EndIndex));
      if WithQuote then
        Prog := Copy(Prog, 2, Length(Prog) - 2)
    end;
  end;

var
  Prog, Parm: String;
  Sei: TShellExecuteInfo;
begin
  Result := 0;

  ExtractCommandLine(CmdLine, Prog, Parm);
  if Prog = '' then Exit;

  FillChar(Sei, SizeOf(Sei), #0);
  Sei.cbSize        := SizeOf(Sei);
  Sei.fMask         := SEE_MASK_DOENVSUBST or SEE_MASK_FLAG_NO_UI;
  Sei.lpFile        := PChar(Prog);
  Sei.lpParameters  := PChar(Parm);
  Sei.lpVerb        := PChar('');
  Sei.nShow         := SW_SHOWNORMAL;

  if not ShellExecuteEx(@Sei) then
    Result := GetLastError;

end;


function AppVer: String;
var
  AppName, Ver: String;
  n, Len: Cardinal;
  Buf, Value: PChar;
begin
  Ver := '0.0.0.0';
  AppName := Application.ExeName;
  n := GetFileVersionInfoSize(PChar(AppName), n);
  if n > 0 then
  begin
    Buf := AllocMem(n);
    GetFileVersionInfo(PChar(AppName), 0, n, Buf);
    if VerQueryValue(Buf, PChar('StringFileInfo\041103a4\FileVersion'), Pointer(Value), Len) then
      Ver := Value;
    FreeMem(Buf, n);
  end;
  Result := Ver;
end;

function StrToRect(Str: String): TRect;
var
  SL: TStringList;
begin
  SL := TStringList.Create;
  try
    SL.CommaText := Str;
    if SL.Count = 4 then
      Result := Rect(StrToInt(SL[0]), StrToInt(SL[1]),
                     StrToInt(SL[2]), StrToInt(SL[3]))
    else
      with Screen do
        Result := Rect((Width - 600) div 2, (Height - 430) div 2, 600, 430);
  finally
    SL.Free;
  end;
end;

function RectToStr(Rect: TRect): String;
begin
  Result := IntToStr(Rect.Left)  + ',' + IntToStr(Rect.Top) + ',' +
            IntToStr(Rect.Right) + ',' + IntToStr(Rect.Bottom);
end;

function GetFileNameByShortCutLnk(LnkName: WideString): String;
var
  hr: HResult;
  ShellLink: IShellLink;
  PersistFile: IPersistFile;
  Path: array[0..MAX_PATH] of Char;
  pfd: TWin32FindData;
begin
  if AnsiUpperCase(ExtractFileExt(LnkName)) <> '.LNK' then
  begin
    Result := LnkName;
    Exit;
  end;
  Result := '';
  hr := CoCreateInstance(CLSID_ShellLink, nil, CLSCTX_INPROC_SERVER,
                         IID_IShellLinkA, ShellLink);
  if Succeeded(hr) then begin
    PersistFile := ShellLink As IPersistFile;
    hr := PersistFile.Load(PWideChar(LnkName), STGM_READ);
    if Succeeded(hr) then begin
      hr := ShellLink.Resolve(Application.Handle, SLR_ANY_MATCH);
      if Succeeded(hr) then begin
        hr := ShellLink.GetPath(Path, MAX_PATH, pfd, SLGP_SHORTPATH);
        if Succeeded(hr) then Result := Path;
      end;
    end;
  end;
end;

function FontStylesToString(Styles: TFontStyles): String;
begin
  Result := '';
  if fsBold in Styles then Result := Result + 'B';
  if fsItalic in Styles then Result := Result + 'I';
  if fsUnderline in Styles then Result := Result + 'U';
  if fsStrikeOut in Styles then Result := Result + 'S';
end;

function StringToFontStyles(Styles: string): TFontStyles;
begin
  Result := [];
  if Pos('B', UpperCase(Styles)) > 0 then Include(Result, fsBold);
  if Pos('I', UpperCase(Styles)) > 0 then Include(Result, fsItalic);
  if Pos('U', UpperCase(Styles)) > 0 then Include(Result, fsUnderline);
  if Pos('S', UpperCase(Styles)) > 0 then Include(Result, fsStrikeOut);
end;

function GetAveCharSize(Canvas: TCanvas): TPoint;
var
  I: Integer;
  Buffer: array[0..51] of Char;
begin
  for I := 0 to 25 do Buffer[I] := Chr(I + Ord('A'));
  for I := 0 to 25 do Buffer[I + 26] := Chr(I + Ord('a'));
  GetTextExtentPoint(Canvas.Handle, Buffer, 52, TSize(Result));
  Result.X := Result.X div 52;
end;

function CenteredInputQuery(const ACaption, APrompt: string; var Value: string; MainForm: TForm): Boolean;
var
  Form: TForm;
  Prompt: TLabel;
  Edit: TEdit;
  DialogUnits: TPoint;
  ButtonTop, ButtonWidth, ButtonHeight: Integer;
begin
  Result := False;
  Form := TForm.Create(Application);
  with Form do
    try
      Canvas.Font := Font;
      DialogUnits := GetAveCharSize(Canvas);
      BorderStyle := bsDialog;
      Caption := ACaption;
      ClientWidth := MulDiv(180, DialogUnits.X, 4);
      ClientHeight := MulDiv(63, DialogUnits.Y, 8);
//      Position := poScreenCenter;
      Prompt := TLabel.Create(Form);
      with Prompt do
      begin
        Parent := Form;
        AutoSize := True;
        Left := MulDiv(8, DialogUnits.X, 4);
        Top := MulDiv(8, DialogUnits.Y, 8);
        Caption := APrompt;
      end;
      Edit := TEdit.Create(Form);
      with Edit do
      begin
        Parent := Form;
        Left := Prompt.Left;
        Top := MulDiv(19, DialogUnits.Y, 8);
        Width := MulDiv(164, DialogUnits.X, 4);
        MaxLength := 255;
        Text := Value;
        SelectAll;
      end;
      ButtonTop := MulDiv(41, DialogUnits.Y, 8);
      ButtonWidth := MulDiv(50, DialogUnits.X, 4);
      ButtonHeight := MulDiv(14, DialogUnits.Y, 8);
      with TButton.Create(Form) do
      begin
        Parent := Form;
        Caption := 'OK';
        ModalResult := mrOk;
        Default := True;
        SetBounds(MulDiv(38, DialogUnits.X, 4), ButtonTop, ButtonWidth,
          ButtonHeight);
      end;
      with TButton.Create(Form) do
      begin
        Parent := Form;
        Caption := 'LZ';
        ModalResult := mrCancel;
        Cancel := True;
        SetBounds(MulDiv(92, DialogUnits.X, 4), ButtonTop, ButtonWidth,
          ButtonHeight);
      end;

      MoveMainFormCenter(Form, MainForm);

      if ShowModal = mrOk then
      begin
        Value := Edit.Text;
        Result := True;
      end;
    finally
      Form.Free;
    end;
end;

function ActivatePrevInstance(const MainFormClass, ATitle: string): HWnd;
var
  BufClass, BufTitle: PChar;
begin
  Result := 0;

  if (MainFormClass = '') and (ATitle = '') then
    Exit;

  BufClass := nil;
  BufTitle := nil;

  if (MainFormClass <> '') then
    BufClass := StrAlloc(Length(MainFormClass) + 1);
  if (ATitle <> '') then
    BufTitle := StrAlloc(Length(ATitle) + 1);
  try
    Result := FindWindow(BufClass, BufTitle);
    SetForegroundWindow(Result);
  finally
    StrDispose(BufTitle);
    StrDispose(BufClass);
  end;
end;

procedure LockForm(AForm: TForm);
begin
  Inc(FMainFormLockCount);
  if FMainFormLockCount = 1 then
  begin
    Screen.Cursor := crHourGlass;
    if AForm.HandleAllocated then
      AForm.Enabled := False;
  end;
end;

procedure UnLockForm(AForm: TForm);
begin
  Dec(FMainFormLockCount);
  if FMainFormLockCount = 0 then
  begin
    AForm.Enabled := True;
    if AForm.HandleAllocated then
      Screen.Cursor := crDefault;
  end;
end;


procedure SetComObject;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    if Reg.OpenKey('\CLSID\{53479C7B-8BC0-4921-B375-3105BBAFBA9D}', True) then
      Reg.WriteString('', 'BinEdit ContextMenu Handler');
    if Reg.OpenKey('\CLSID\{53479C7B-8BC0-4921-B375-3105BBAFBA9D}\InProcServer32', True) then 
    begin
      Reg.WriteString('', AppPath + 'BESHELLX.DLL');
      Reg.WriteString('ThreadingModel', 'Apartment');
    end;
    if Reg.OpenKey('\*\shellex\ContextMenuHandlers\BinEdit', True) then
      Reg.WriteString('', '{53479C7B-8BC0-4921-B375-3105BBAFBA9D}');
  finally
    Reg.Free;
  end;
end;

procedure DelComObject;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    Reg.DeleteKey('\CLSID\{53479C7B-8BC0-4921-B375-3105BBAFBA9D}');
    Reg.DeleteKey('\*\shellex\ContextMenuHandlers\BinEdit');
  finally
    Reg.Free;
  end;
end;

function UsingShellExtention: Boolean;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    Result := Reg.KeyExists('\CLSID\{53479C7B-8BC0-4921-B375-3105BBAFBA9D}');
  finally
    Reg.Free;
  end;
end;

end.
